import java.awt.geom.Point2D;

public class TrialTrace
{
	final float TWO_TIMES_PI = 6.283185307f;

	int sequence, trialNumber;
	double a, w; // target amplitude, target width
	double fromX, fromY, targetX, targetY;
	double selectX, selectY;
	int positioningTime, selectionTime, movementTime;
	int ballDiameter; // diameter of the ball that is moved to a target

	TracePoint[] p; // raw trace points

	AccuracyMeasures ac; // 7 accuracy measures (see below)
	int tre; // target re-entries
	int tac; // task axis crossings
	int mdc; // movement direction changes
	int odc; // orthogonal direction changes
	double mv; // movement variability
	double me; // movement error
	double mo; // movement offset
	int numberOfSamplePoints;
	
	double maxTilt; // maximum tilt angle (degrees) during a trial

	TrialTrace(int sequenceArg, int trialNumberArg, double aArg, double wArg, double fromXArg, double fromYArg, double targetXArg,
			double targetYArg, int ballDiameterArg, TracePoint[] pArg)
	{
		sequence = sequenceArg;
		trialNumber = trialNumberArg;
		a = aArg;
		w = wArg;
		fromX = fromXArg;
		fromY = fromYArg;
		targetX = targetXArg;
		targetY = targetYArg;
		ballDiameter = ballDiameterArg;
		p = pArg;

		selectX = p[p.length - 1].x;
		selectY = p[p.length - 1].y;

		// do things that require iterating over the trace points
		Point2D.Double[] path = new Point2D.Double[p.length];
		int start = p[0].timeStamp;
		movementTime = p[p.length - 1].timeStamp - start;
		boolean ballEnteredTarget = false;
		for (int i = 0; i < path.length; ++i)
		{
			// create a new array of Point2D.Double's (for AccuracyMeasures)
			path[i] = new Point2D.Double(p[i].x, p[i].y);
			
			// find the maximum tilt
			if (p[i].tilt > maxTilt)
				maxTilt = p[i].tilt;
			
			// calculated positioning time (time to first-entry)
			double distanceFromTarget = Math.hypot(p[i].x - targetX, p[i].y - targetY);
			if (!ballEnteredTarget && distanceFromTarget < (w / 2.0 + ballDiameter / 2.0))
			{
				ballEnteredTarget = true;
				positioningTime = p[i].timeStamp - start;				
			}
		}
		selectionTime = movementTime - positioningTime;		

		ac = new AccuracyMeasures(new Point2D.Double(fromX, fromY), new Point2D.Double(targetX, targetY), w, path);
		tre = ac.getTRE();
		tac = ac.getTAC();
		mdc = ac.getMDC();
		odc = ac.getODC();
		mv = ac.getMV();
		me = ac.getME();
		mo = ac.getMO();		
		numberOfSamplePoints = path.length;
	}

	public String tPoints()
	{
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < p.length; ++i)
			sb.append(p[i].timeStamp + ",");
		sb.deleteCharAt(sb.length() - 1); // delete last comma
		return sb.toString();
	}

	public String xPoints()
	{
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < p.length; ++i)
			sb.append(p[i].x + ",");
		sb.deleteCharAt(sb.length() - 1); // delete last comma
		return sb.toString();
	}

	public String yPoints()
	{
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < p.length; ++i)
			sb.append(p[i].y + ",");
		sb.deleteCharAt(sb.length() - 1); // delete last comma
		return sb.toString();
	}

	public String tiltPoints()
	{
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < p.length; ++i)
			sb.append(p[i].tilt + ",");
		sb.deleteCharAt(sb.length() - 1); // delete last comma
		return sb.toString();
	}	
}
